@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 """
     sphinx.ext.apidoc
     ~~~~~~~~~~~~~~~~~
@@ -9,30 +8,32 @@
 
     This is derived from the "sphinx-autopackage" script, which is:
     Copyright 2008 Société des arts technologiques (SAT),
-    http://www.sat.qc.ca/
+    https://sat.qc.ca/
 
-    :copyright: Copyright 2007-2018 by the Sphinx team, see AUTHORS.
+    :copyright: Copyright 2007-2019 by the Sphinx team, see AUTHORS.
     :license: BSD, see LICENSE for details.
 """
 
-from __future__ import print_function
-
 import argparse
 import glob
+import locale
 import os
 import sys
+import warnings
 from fnmatch import fnmatch
 from os import path
-from typing import TYPE_CHECKING
 
-from six import binary_type
-
-from sphinx import __display_version__
+import sphinx.locale
+from sphinx import __display_version__, package_dir
 from sphinx.cmd.quickstart import EXTENSIONS
+from sphinx.deprecation import RemovedInSphinx40Warning
+from sphinx.locale import __
 from sphinx.util import rst
-from sphinx.util.osutil import FileAvoidWrite, ensuredir, walk
+from sphinx.util.osutil import FileAvoidWrite, ensuredir
+from sphinx.util.template import ReSTRenderer
 
-if TYPE_CHECKING:
+if False:
+    # For type annotation
     from typing import Any, List, Tuple  # NOQA
 
 # automodule options
@@ -47,12 +48,16 @@ else:
     ]
 
 INITPY = '__init__.py'
-PY_SUFFIXES = set(['.py', '.pyx'])
+PY_SUFFIXES = {'.py', '.pyx'}
+
+template_dir = path.join(package_dir, 'templates', 'apidoc')
 
 
 def makename(package, module):
-    # type: (unicode, unicode) -> unicode
+    # type: (str, str) -> str
     """Join package and module with a dot."""
+    warnings.warn('makename() is deprecated.',
+                  RemovedInSphinx40Warning)
     # Both package and module can be None/empty.
     if package:
         name = package
@@ -63,24 +68,32 @@ def makename(package, module):
     return name
 
 
+def module_join(*modnames):
+    # type: (*str) -> str
+    """Join module names with dots."""
+    return '.'.join(filter(None, modnames))
+
+
 def write_file(name, text, opts):
-    # type: (unicode, unicode, Any) -> None
+    # type: (str, str, Any) -> None
     """Write the output file for module/package <name>."""
     fname = path.join(opts.destdir, '%s.%s' % (name, opts.suffix))
     if opts.dryrun:
-        print('Would create file %s.' % fname)
+        print(__('Would create file %s.') % fname)
         return
     if not opts.force and path.isfile(fname):
-        print('File %s already exists, skipping.' % fname)
+        print(__('File %s already exists, skipping.') % fname)
     else:
-        print('Creating file %s.' % fname)
+        print(__('Creating file %s.') % fname)
         with FileAvoidWrite(fname) as f:
             f.write(text)
 
 
 def format_heading(level, text, escape=True):
-    # type: (int, unicode, bool) -> unicode
+    # type: (int, str, bool) -> str
     """Create a heading of <level> [1, 2 or 3 supported]."""
+    warnings.warn('format_warning() is deprecated.',
+                  RemovedInSphinx40Warning)
     if escape:
         text = rst.escape(text)
     underlining = ['=', '-', '~', ][level - 1] * len(text)
@@ -88,122 +101,102 @@ def format_heading(level, text, escape=True):
 
 
 def format_directive(module, package=None):
-    # type: (unicode, unicode) -> unicode
+    # type: (str, str) -> str
     """Create the automodule directive and add the options."""
-    directive = '.. automodule:: %s\n' % makename(package, module)
+    warnings.warn('format_directive() is deprecated.',
+                  RemovedInSphinx40Warning)
+    directive = '.. automodule:: %s\n' % module_join(package, module)
     for option in OPTIONS:
         directive += '    :%s:\n' % option
     return directive
 
 
-def create_module_file(package, module, opts):
-    # type: (unicode, unicode, Any) -> None
+def create_module_file(package, basename, opts):
+    # type: (str, str, Any) -> None
     """Build the text of the file and write the file."""
-    if not opts.noheadings:
-        text = format_heading(1, '%s module' % module)
-    else:
-        text = ''
-    # text += format_heading(2, ':mod:`%s` Module' % module)
-    text += format_directive(module, package)
-    write_file(makename(package, module), text, opts)
+    qualname = module_join(package, basename)
+    context = {
+        'show_headings': not opts.noheadings,
+        'basename': basename,
+        'qualname': qualname,
+        'automodule_options': OPTIONS,
+    }
+    text = ReSTRenderer(template_dir).render('module.rst', context)
+    write_file(qualname, text, opts)
 
 
 def create_package_file(root, master_package, subroot, py_files, opts, subs, is_namespace, excludes=[]):  # NOQA
-    # type: (unicode, unicode, unicode, List[unicode], Any, List[unicode], bool, List[unicode]) -> None  # NOQA
+    # type: (str, str, str, List[str], Any, List[str], bool, List[str]) -> None
     """Build the text of the file and write the file."""
-    text = format_heading(1, ('%s package' if not is_namespace else "%s namespace")
-                          % makename(master_package, subroot))
+    # build a list of sub packages (directories containing an INITPY file)
+    subpackages = [sub for sub in subs if not
+                   shall_skip(path.join(root, sub, INITPY), opts, excludes)]
+    subpackages = [module_join(master_package, subroot, pkgname)
+                   for pkgname in subpackages]
+    # build a list of sub modules
+    submodules = [path.splitext(sub)[0] for sub in py_files
+                  if not is_skipped_module(path.join(root, sub), opts, excludes) and
+                  sub != INITPY]
+    submodules = [module_join(master_package, subroot, modname)
+                  for modname in submodules]
 
-    if opts.modulefirst and not is_namespace:
-        text += format_directive(subroot, master_package)
-        text += '\n'
+    pkgname = module_join(master_package, subroot)
+    context = {
+        'pkgname': pkgname,
+        'subpackages': subpackages,
+        'submodules': submodules,
+        'is_namespace': is_namespace,
+        'modulefirst': opts.modulefirst,
+        'separatemodules': opts.separatemodules,
+        'automodule_options': OPTIONS,
+        'show_headings': not opts.noheadings,
+    }
+    text = ReSTRenderer(template_dir).render('package.rst', context)
+    write_file(pkgname, text, opts)
 
-    # build a list of directories that are szvpackages (contain an INITPY file)
-    # and also checks the INITPY file is not empty, or there are other python
-    # source files in that folder.
-    # (depending on settings - but shall_skip() takes care of that)
-    subs = [sub for sub in subs if not
-            shall_skip(path.join(root, sub, INITPY), opts, excludes)]
-    # if there are some package directories, add a TOC for theses subpackages
-    if subs:
-        text += format_heading(2, 'Subpackages')
-        text += '.. toctree::\n\n'
-        for sub in subs:
-            text += '    %s.%s\n' % (makename(master_package, subroot), sub)
-        text += '\n'
-
-    submods = [path.splitext(sub)[0] for sub in py_files
-               if not shall_skip(path.join(root, sub), opts, excludes) and
-               sub != INITPY]
-    if submods:
-        text += format_heading(2, 'Submodules')
-        if opts.separatemodules:
-            text += '.. toctree::\n\n'
-            for submod in submods:
-                modfile = makename(master_package, makename(subroot, submod))
-                text += '   %s\n' % modfile
-
-                # generate separate file for this module
-                if not opts.noheadings:
-                    filetext = format_heading(1, '%s module' % modfile)
-                else:
-                    filetext = ''
-                filetext += format_directive(makename(subroot, submod),
-                                             master_package)
-                write_file(modfile, filetext, opts)
-        else:
-            for submod in submods:
-                modfile = makename(master_package, makename(subroot, submod))
-                if not opts.noheadings:
-                    text += format_heading(2, '%s module' % modfile)
-                text += format_directive(makename(subroot, submod),
-                                         master_package)
-                text += '\n'
-        text += '\n'
-
-    if not opts.modulefirst and not is_namespace:
-        text += format_heading(2, 'Module contents')
-        text += format_directive(subroot, master_package)
-
-    write_file(makename(master_package, subroot), text, opts)
+    if submodules and opts.separatemodules:
+        for submodule in submodules:
+            create_module_file(None, submodule, opts)
 
 
 def create_modules_toc_file(modules, opts, name='modules'):
-    # type: (List[unicode], Any, unicode) -> None
+    # type: (List[str], Any, str) -> None
     """Create the module's index."""
-    text = format_heading(1, '%s' % opts.header, escape=False)
-    text += '.. toctree::\n'
-    text += '   :maxdepth: %s\n\n' % opts.maxdepth
-
     modules.sort()
-    prev_module = ''  # type: unicode
-    for module in modules:
+    prev_module = ''
+    for module in modules[:]:
         # look if the module is a subpackage and, if yes, ignore it
         if module.startswith(prev_module + '.'):
-            continue
-        prev_module = module
-        text += '   %s\n' % module
+            modules.remove(module)
+        else:
+            prev_module = module
 
+    context = {
+        'header': opts.header,
+        'maxdepth': opts.maxdepth,
+        'docnames': modules,
+    }
+    text = ReSTRenderer(template_dir).render('toc.rst', context)
     write_file(name, text, opts)
 
 
 def shall_skip(module, opts, excludes=[]):
-    # type: (unicode, Any, List[unicode]) -> bool
+    # type: (str, Any, List[str]) -> bool
     """Check if we want to skip this module."""
     # skip if the file doesn't exist and not using implicit namespaces
     if not opts.implicit_namespaces and not path.exists(module):
         return True
 
-    # skip it if there is nothing (or just \n or \r\n) in the file
-    if path.exists(module) and path.getsize(module) <= 2:
-        if os.path.basename(module) == '__init__.py':
-            # We only want to skip packages if they do not contain any
-            # .py files other than __init__.py.
-            basemodule = path.dirname(module)
-            for module in glob.glob(path.join(basemodule, '*.py')):
-                if not is_excluded(path.join(basemodule, module), excludes):
-                    return True
-        else:
+    # Are we a package (here defined as __init__.py, not the folder in itself)
+    if os.path.basename(module) == INITPY:
+        # Yes, check if we have any non-excluded modules at all here
+        all_skipped = True
+        basemodule = path.dirname(module)
+        for submodule in glob.glob(path.join(basemodule, '*.py')):
+            if not is_excluded(path.join(basemodule, submodule), excludes):
+                # There's a non-excluded module here, we won't skip
+                all_skipped = False
+        if all_skipped:
             return True
 
     # skip if it has a "private" name and this is selected
@@ -214,8 +207,21 @@ def shall_skip(module, opts, excludes=[]):
     return False
 
 
+def is_skipped_module(filename, opts, excludes):
+    # type: (str, Any, List[str]) -> bool
+    """Check if we want to skip this module."""
+    if not path.exists(filename):
+        # skip if the file doesn't exist
+        return True
+    elif path.basename(filename).startswith('_') and not opts.includeprivate:
+        # skip if the module has a "private" name
+        return True
+    else:
+        return False
+
+
 def recurse_tree(rootpath, excludes, opts):
-    # type: (unicode, List[unicode], Any) -> List[unicode]
+    # type: (str, List[str], Any) -> List[str]
     """
     Look for every file in the directory tree and create the corresponding
     ReST files.
@@ -232,7 +238,7 @@ def recurse_tree(rootpath, excludes, opts):
         root_package = None
 
     toplevels = []
-    for root, subs, files in walk(rootpath, followlinks=followlinks):
+    for root, subs, files in os.walk(rootpath, followlinks=followlinks):
         # document only Python module files (that aren't excluded)
         py_files = sorted(f for f in files
                           if path.splitext(f)[1] in PY_SUFFIXES and
@@ -250,7 +256,7 @@ def recurse_tree(rootpath, excludes, opts):
         # remove hidden ('.') and private ('_') directories, as well as
         # excluded dirs
         if includeprivate:
-            exclude_prefixes = ('.',)  # type: Tuple[unicode, ...]
+            exclude_prefixes = ('.',)  # type: Tuple[str, ...]
         else:
             exclude_prefixes = ('.', '_')
         subs[:] = sorted(sub for sub in subs if not sub.startswith(exclude_prefixes) and
@@ -266,12 +272,12 @@ def recurse_tree(rootpath, excludes, opts):
                 if not is_namespace or len(py_files) > 0:
                     create_package_file(root, root_package, subpackage,
                                         py_files, opts, subs, is_namespace, excludes)
-                    toplevels.append(makename(root_package, subpackage))
+                    toplevels.append(module_join(root_package, subpackage))
         else:
             # if we are at the root level, we don't require it to be a package
             assert root == rootpath and root_package is None
             for py_file in py_files:
-                if not shall_skip(path.join(rootpath, py_file), opts):
+                if not is_skipped_module(path.join(rootpath, py_file), opts, excludes):
                     module = path.splitext(py_file)[0]
                     create_module_file(root_package, module, opts)
                     toplevels.append(module)
@@ -280,11 +286,11 @@ def recurse_tree(rootpath, excludes, opts):
 
 
 def is_excluded(root, excludes):
-    # type: (unicode, List[unicode]) -> bool
+    # type: (str, List[str]) -> bool
     """Check if the directory is in the exclude list.
 
     Note: by having trailing slashes, we avoid common prefix issues, like
-          e.g. an exlude "foo" also accidentally excluding "foobar".
+          e.g. an exclude "foo" also accidentally excluding "foobar".
     """
     for exclude in excludes:
         if fnmatch(root, exclude):
@@ -295,86 +301,90 @@ def is_excluded(root, excludes):
 def get_parser():
     # type: () -> argparse.ArgumentParser
     parser = argparse.ArgumentParser(
-        usage='usage: %(prog)s [OPTIONS] -o <OUTPUT_PATH> <MODULE_PATH> '
+        usage='%(prog)s [OPTIONS] -o <OUTPUT_PATH> <MODULE_PATH> '
               '[EXCLUDE_PATTERN, ...]',
-        epilog='For more information, visit <http://sphinx-doc.org/>.',
-        description="""
+        epilog=__('For more information, visit <http://sphinx-doc.org/>.'),
+        description=__("""
 Look recursively in <MODULE_PATH> for Python modules and packages and create
 one reST file with automodule directives per package in the <OUTPUT_PATH>.
 
 The <EXCLUDE_PATTERN>s can be file and/or directory patterns that will be
 excluded from generation.
 
-Note: By default this script will not overwrite already created files.""")
+Note: By default this script will not overwrite already created files."""))
 
     parser.add_argument('--version', action='version', dest='show_version',
                         version='%%(prog)s %s' % __display_version__)
 
     parser.add_argument('module_path',
-                        help='path to module to document')
+                        help=__('path to module to document'))
     parser.add_argument('exclude_pattern', nargs='*',
-                        help='fnmatch-style file and/or directory patterns '
-                        'to exclude from generation')
+                        help=__('fnmatch-style file and/or directory patterns '
+                                'to exclude from generation'))
 
     parser.add_argument('-o', '--output-dir', action='store', dest='destdir',
                         required=True,
-                        help='directory to place all output')
+                        help=__('directory to place all output'))
     parser.add_argument('-d', '--maxdepth', action='store', dest='maxdepth',
                         type=int, default=4,
-                        help='maximum depth of submodules to show in the TOC '
-                             '(default: 4)')
+                        help=__('maximum depth of submodules to show in the TOC '
+                                '(default: 4)'))
     parser.add_argument('-f', '--force', action='store_true', dest='force',
-                        help='overwrite existing files')
+                        help=__('overwrite existing files'))
     parser.add_argument('-l', '--follow-links', action='store_true',
                         dest='followlinks', default=False,
-                        help='follow symbolic links. Powerful when combined '
-                              'with collective.recipe.omelette.')
+                        help=__('follow symbolic links. Powerful when combined '
+                                'with collective.recipe.omelette.'))
     parser.add_argument('-n', '--dry-run', action='store_true', dest='dryrun',
-                        help='run the script without creating files')
+                        help=__('run the script without creating files'))
     parser.add_argument('-e', '--separate', action='store_true',
                         dest='separatemodules',
-                        help='put documentation for each module on its own page')
+                        help=__('put documentation for each module on its own page'))
     parser.add_argument('-P', '--private', action='store_true',
                         dest='includeprivate',
-                        help='include "_private" modules')
-    parser.add_argument('-T', '--no-toc', action='store_true', dest='notoc',
-                        help="don't create a table of contents file")
+                        help=__('include "_private" modules'))
+    parser.add_argument('--tocfile', action='store', dest='tocfile', default='modules',
+                        help=__("filename of table of contents (default: modules)"))
+    parser.add_argument('-T', '--no-toc', action='store_false', dest='tocfile',
+                        help=__("don't create a table of contents file"))
     parser.add_argument('-E', '--no-headings', action='store_true',
                         dest='noheadings',
-                        help="don't create headings for the module/package "
-                             "packages (e.g. when the docstrings already "
-                             "contain them)")
+                        help=__("don't create headings for the module/package "
+                                "packages (e.g. when the docstrings already "
+                                "contain them)"))
     parser.add_argument('-M', '--module-first', action='store_true',
                         dest='modulefirst',
-                        help='put module documentation before submodule '
-                             'documentation')
+                        help=__('put module documentation before submodule '
+                                'documentation'))
     parser.add_argument('--implicit-namespaces', action='store_true',
                         dest='implicit_namespaces',
-                        help='interpret module paths according to PEP-0420 '
-                             'implicit namespaces specification')
+                        help=__('interpret module paths according to PEP-0420 '
+                                'implicit namespaces specification'))
     parser.add_argument('-s', '--suffix', action='store', dest='suffix',
                         default='rst',
-                        help='file suffix (default: rst)')
+                        help=__('file suffix (default: rst)'))
     parser.add_argument('-F', '--full', action='store_true', dest='full',
-                        help='generate a full project with sphinx-quickstart')
+                        help=__('generate a full project with sphinx-quickstart'))
     parser.add_argument('-a', '--append-syspath', action='store_true',
                         dest='append_syspath',
-                        help='append module_path to sys.path, used when --full is given')
+                        help=__('append module_path to sys.path, used when --full is given'))
     parser.add_argument('-H', '--doc-project', action='store', dest='header',
-                        help='project name (default: root module name)')
+                        help=__('project name (default: root module name)'))
     parser.add_argument('-A', '--doc-author', action='store', dest='author',
-                        help='project author(s), used when --full is given')
+                        help=__('project author(s), used when --full is given'))
     parser.add_argument('-V', '--doc-version', action='store', dest='version',
-                        help='project version, used when --full is given')
+                        help=__('project version, used when --full is given'))
     parser.add_argument('-R', '--doc-release', action='store', dest='release',
-                        help='project release, used when --full is given, '
-                             'defaults to --doc-version')
+                        help=__('project release, used when --full is given, '
+                                'defaults to --doc-version'))
 
-    group = parser.add_argument_group('extension options')
+    group = parser.add_argument_group(__('extension options'))
+    group.add_argument('--extensions', metavar='EXTENSIONS', dest='extensions',
+                       action='append', help=__('enable arbitrary extensions'))
     for ext in EXTENSIONS:
         group.add_argument('--ext-%s' % ext, action='append_const',
                            const='sphinx.ext.%s' % ext, dest='extensions',
-                           help='enable %s extension' % ext)
+                           help=__('enable %s extension') % ext)
 
     return parser
 
@@ -382,6 +392,9 @@ Note: By default this script will not overwrite already created files.""")
 def main(argv=sys.argv[1:]):
     # type: (List[str]) -> int
     """Parse and check the command line arguments."""
+    sphinx.locale.setlocale(locale.LC_ALL, '')
+    sphinx.locale.init_console(os.path.join(package_dir, 'locale'), 'sphinx')
+
     parser = get_parser()
     args = parser.parse_args(argv)
 
@@ -394,7 +407,7 @@ def main(argv=sys.argv[1:]):
     if args.suffix.startswith('.'):
         args.suffix = args.suffix[1:]
     if not path.isdir(rootpath):
-        print('%s is not a directory.' % rootpath, file=sys.stderr)
+        print(__('%s is not a directory.') % rootpath, file=sys.stderr)
         sys.exit(1)
     if not args.dryrun:
         ensuredir(args.destdir)
@@ -404,51 +417,47 @@ def main(argv=sys.argv[1:]):
     if args.full:
         from sphinx.cmd import quickstart as qs
         modules.sort()
-        prev_module = ''  # type: unicode
+        prev_module = ''
         text = ''
         for module in modules:
             if module.startswith(prev_module + '.'):
                 continue
             prev_module = module
             text += '   %s\n' % module
-        d = dict(
-            path = args.destdir,
-            sep = False,
-            dot = '_',
-            project = args.header,
-            author = args.author or 'Author',
-            version = args.version or '',
-            release = args.release or args.version or '',
-            suffix = '.' + args.suffix,
-            master = 'index',
-            epub = True,
-            extensions = ['sphinx.ext.autodoc', 'sphinx.ext.viewcode',
-                          'sphinx.ext.todo'],
-            makefile = True,
-            batchfile = True,
-            make_mode = True,
-            mastertocmaxdepth = args.maxdepth,
-            mastertoctree = text,
-            language = 'en',
-            module_path = rootpath,
-            append_syspath = args.append_syspath,
-        )
+        d = {
+            'path': args.destdir,
+            'sep': False,
+            'dot': '_',
+            'project': args.header,
+            'author': args.author or 'Author',
+            'version': args.version or '',
+            'release': args.release or args.version or '',
+            'suffix': '.' + args.suffix,
+            'master': 'index',
+            'epub': True,
+            'extensions': ['sphinx.ext.autodoc', 'sphinx.ext.viewcode',
+                           'sphinx.ext.todo'],
+            'makefile': True,
+            'batchfile': True,
+            'make_mode': True,
+            'mastertocmaxdepth': args.maxdepth,
+            'mastertoctree': text,
+            'language': 'en',
+            'module_path': rootpath,
+            'append_syspath': args.append_syspath,
+        }
         if args.extensions:
             d['extensions'].extend(args.extensions)
 
-        if isinstance(args.header, binary_type):
-            d['project'] = d['project'].decode('utf-8')
-        if isinstance(args.author, binary_type):
-            d['author'] = d['author'].decode('utf-8')
-        if isinstance(args.version, binary_type):
-            d['version'] = d['version'].decode('utf-8')
-        if isinstance(args.release, binary_type):
-            d['release'] = d['release'].decode('utf-8')
+        for ext in d['extensions'][:]:
+            if ',' in ext:
+                d['extensions'].remove(ext)
+                d['extensions'].extend(ext.split(','))
 
         if not args.dryrun:
             qs.generate(d, silent=True, overwrite=args.force)
-    elif not args.notoc:
-        create_modules_toc_file(modules, args)
+    elif args.tocfile:
+        create_modules_toc_file(modules, args, args.tocfile)
 
     return 0
 
