@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 """
     sphinx.ext.autosummary.generate
     ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
@@ -14,40 +13,39 @@
        generate:
                sphinx-autogen -o source/generated source/*.rst
 
-    :copyright: Copyright 2007-2018 by the Sphinx team, see AUTHORS.
+    :copyright: Copyright 2007-2019 by the Sphinx team, see AUTHORS.
     :license: BSD, see LICENSE for details.
 """
-from __future__ import print_function
 
 import argparse
-import codecs
+import locale
 import os
 import pydoc
 import re
 import sys
-from typing import TYPE_CHECKING
 
-from jinja2 import FileSystemLoader, TemplateNotFound
+from jinja2 import BaseLoader, FileSystemLoader, TemplateNotFound
 from jinja2.sandbox import SandboxedEnvironment
 
+import sphinx.locale
 from sphinx import __display_version__
 from sphinx import package_dir
 from sphinx.ext.autosummary import import_by_name, get_documenter
 from sphinx.jinja2glue import BuiltinTemplateLoader
+from sphinx.locale import __
 from sphinx.registry import SphinxComponentRegistry
+from sphinx.util import rst
 from sphinx.util.inspect import safe_getattr
 from sphinx.util.osutil import ensuredir
-from sphinx.util.rst import escape as rst_escape
 
-if TYPE_CHECKING:
-    from typing import Any, Callable, Dict, Tuple, List  # NOQA
-    from jinja2 import BaseLoader  # NOQA
-    from sphinx import addnodes  # NOQA
+if False:
+    # For type annotation
+    from typing import Any, Callable, Dict, List, Set, Tuple, Type, Union  # NOQA
     from sphinx.builders import Builder  # NOQA
-    from sphinx.environment import BuildEnvironment  # NOQA
+    from sphinx.ext.autodoc import Documenter  # NOQA
 
 
-class DummyApplication(object):
+class DummyApplication:
     """Dummy Application class for sphinx-autogen command."""
 
     def __init__(self):
@@ -60,73 +58,93 @@ def setup_documenters(app):
     from sphinx.ext.autodoc import (
         ModuleDocumenter, ClassDocumenter, ExceptionDocumenter, DataDocumenter,
         FunctionDocumenter, MethodDocumenter, AttributeDocumenter,
-        InstanceAttributeDocumenter
+        InstanceAttributeDocumenter, DecoratorDocumenter, PropertyDocumenter,
+        SlotsAttributeDocumenter,
     )
     documenters = [
         ModuleDocumenter, ClassDocumenter, ExceptionDocumenter, DataDocumenter,
         FunctionDocumenter, MethodDocumenter, AttributeDocumenter,
-        InstanceAttributeDocumenter
-    ]
+        InstanceAttributeDocumenter, DecoratorDocumenter, PropertyDocumenter,
+        SlotsAttributeDocumenter,
+    ]  # type: List[Type[Documenter]]
     for documenter in documenters:
         app.registry.add_documenter(documenter.objtype, documenter)
 
 
 def _simple_info(msg):
-    # type: (unicode) -> None
+    # type: (str) -> None
     print(msg)
 
 
 def _simple_warn(msg):
-    # type: (unicode) -> None
+    # type: (str) -> None
     print('WARNING: ' + msg, file=sys.stderr)
 
 
 def _underline(title, line='='):
+    # type: (str, str) -> str
     if '\n' in title:
         raise ValueError('Can only underline single lines')
     return title + '\n' + line * len(title)
 
 
+class AutosummaryRenderer:
+    """A helper class for rendering."""
+
+    def __init__(self, builder, template_dir):
+        # type: (Builder, str) -> None
+        loader = None  # type: BaseLoader
+        template_dirs = [os.path.join(package_dir, 'ext', 'autosummary', 'templates')]
+        if builder is None:
+            if template_dir:
+                template_dirs.insert(0, template_dir)
+            loader = FileSystemLoader(template_dirs)
+        else:
+            # allow the user to override the templates
+            loader = BuiltinTemplateLoader()
+            loader.init(builder, dirs=template_dirs)
+
+        self.env = SandboxedEnvironment(loader=loader)
+        self.env.filters['escape'] = rst.escape
+        self.env.filters['e'] = rst.escape
+        self.env.filters['underline'] = _underline
+
+    def exists(self, template_name):
+        # type: (str) -> bool
+        """Check if template file exists."""
+        try:
+            self.env.get_template(template_name)
+            return True
+        except TemplateNotFound:
+            return False
+
+    def render(self, template_name, context):
+        # type: (str, Dict) -> str
+        """Render a template file."""
+        return self.env.get_template(template_name).render(context)
+
+
 # -- Generating output ---------------------------------------------------------
 
 def generate_autosummary_docs(sources, output_dir=None, suffix='.rst',
                               warn=_simple_warn, info=_simple_info,
                               base_path=None, builder=None, template_dir=None,
                               imported_members=False, app=None):
-    # type: (List[unicode], unicode, unicode, Callable, Callable, unicode, Builder, unicode, bool, Any) -> None  # NOQA
+    # type: (List[str], str, str, Callable, Callable, str, Builder, str, bool, Any) -> None
 
     showed_sources = list(sorted(sources))
     if len(showed_sources) > 20:
         showed_sources = showed_sources[:10] + ['...'] + showed_sources[-10:]
-    info('[autosummary] generating autosummary for: %s' %
+    info(__('[autosummary] generating autosummary for: %s') %
          ', '.join(showed_sources))
 
     if output_dir:
-        info('[autosummary] writing to %s' % output_dir)
+        info(__('[autosummary] writing to %s') % output_dir)
 
     if base_path is not None:
         sources = [os.path.join(base_path, filename) for filename in sources]
 
-    # create our own templating environment
-    template_dirs = None  # type: List[unicode]
-    template_dirs = [os.path.join(package_dir, 'ext',
-                                  'autosummary', 'templates')]
-
-    template_loader = None  # type: BaseLoader
-    if builder is not None:
-        # allow the user to override the templates
-        template_loader = BuiltinTemplateLoader()
-        template_loader.init(builder, dirs=template_dirs)
-    else:
-        if template_dir:
-            template_dirs.insert(0, template_dir)
-        template_loader = FileSystemLoader(template_dirs)  # type: ignore
-    template_env = SandboxedEnvironment(loader=template_loader)
-    template_env.filters['underline'] = _underline
-
-    # replace the builtin html filters
-    template_env.filters['escape'] = rst_escape
-    template_env.filters['e'] = rst_escape
+    template = AutosummaryRenderer(builder, template_dir)
 
     # read
     items = find_autosummary_in_files(sources)
@@ -161,25 +179,21 @@ def generate_autosummary_docs(sources, output_dir=None, suffix='.rst',
         with open(fn, 'w') as f:
             doc = get_documenter(app, obj, parent)
 
-            if template_name is not None:
-                template = template_env.get_template(template_name)
-            else:
-                try:
-                    template = template_env.get_template('autosummary/%s.rst'
-                                                         % doc.objtype)
-                except TemplateNotFound:
-                    template = template_env.get_template('autosummary/base.rst')
+            if template_name is None:
+                template_name = 'autosummary/%s.rst' % doc.objtype
+                if not template.exists(template_name):
+                    template_name = 'autosummary/base.rst'
 
-            def get_members(obj, typ, include_public=[], imported=True):
-                # type: (Any, unicode, List[unicode], bool) -> Tuple[List[unicode], List[unicode]]  # NOQA
-                items = []  # type: List[unicode]
+            def get_members(obj, types, include_public=[], imported=True):
+                # type: (Any, Set[str], List[str], bool) -> Tuple[List[str], List[str]]  # NOQA
+                items = []  # type: List[str]
                 for name in dir(obj):
                     try:
                         value = safe_getattr(obj, name)
                     except AttributeError:
                         continue
                     documenter = get_documenter(app, value, obj)
-                    if documenter.objtype == typ:
+                    if documenter.objtype in types:
                         if imported or getattr(value, '__module__', None) == obj.__name__:
                             # skip imported members if expected
                             items.append(name)
@@ -187,22 +201,24 @@ def generate_autosummary_docs(sources, output_dir=None, suffix='.rst',
                           if x in include_public or not x.startswith('_')]
                 return public, items
 
-            ns = {}  # type: Dict[unicode, Any]
+            ns = {}  # type: Dict[str, Any]
 
             if doc.objtype == 'module':
                 ns['members'] = dir(obj)
                 ns['functions'], ns['all_functions'] = \
-                    get_members(obj, 'function', imported=imported_members)
+                    get_members(obj, {'function'}, imported=imported_members)
                 ns['classes'], ns['all_classes'] = \
-                    get_members(obj, 'class', imported=imported_members)
+                    get_members(obj, {'class'}, imported=imported_members)
                 ns['exceptions'], ns['all_exceptions'] = \
-                    get_members(obj, 'exception', imported=imported_members)
+                    get_members(obj, {'exception'}, imported=imported_members)
             elif doc.objtype == 'class':
                 ns['members'] = dir(obj)
+                ns['inherited_members'] = \
+                    set(dir(obj)) - set(obj.__dict__.keys())
                 ns['methods'], ns['all_methods'] = \
-                    get_members(obj, 'method', ['__init__'])
+                    get_members(obj, {'method'}, ['__init__'])
                 ns['attributes'], ns['all_attributes'] = \
-                    get_members(obj, 'attribute')
+                    get_members(obj, {'attribute', 'property'})
 
             parts = name.split('.')
             if doc.objtype in ('method', 'attribute'):
@@ -221,8 +237,8 @@ def generate_autosummary_docs(sources, output_dir=None, suffix='.rst',
             ns['objtype'] = doc.objtype
             ns['underline'] = len(name) * '='
 
-            rendered = template.render(**ns)
-            f.write(rendered)  # type: ignore
+            rendered = template.render(template_name, ns)
+            f.write(rendered)
 
     # descend recursively to new files
     if new_files:
@@ -235,22 +251,21 @@ def generate_autosummary_docs(sources, output_dir=None, suffix='.rst',
 # -- Finding documented entries in files ---------------------------------------
 
 def find_autosummary_in_files(filenames):
-    # type: (List[unicode]) -> List[Tuple[unicode, unicode, unicode]]
+    # type: (List[str]) -> List[Tuple[str, str, str]]
     """Find out what items are documented in source/*.rst.
 
     See `find_autosummary_in_lines`.
     """
-    documented = []  # type: List[Tuple[unicode, unicode, unicode]]
+    documented = []  # type: List[Tuple[str, str, str]]
     for filename in filenames:
-        with codecs.open(filename, 'r', encoding='utf-8',  # type: ignore
-                         errors='ignore') as f:
+        with open(filename, encoding='utf-8', errors='ignore') as f:
             lines = f.read().splitlines()
             documented.extend(find_autosummary_in_lines(lines, filename=filename))
     return documented
 
 
 def find_autosummary_in_docstring(name, module=None, filename=None):
-    # type: (unicode, Any, unicode) -> List[Tuple[unicode, unicode, unicode]]
+    # type: (str, Any, str) -> List[Tuple[str, str, str]]
     """Find out what items are documented in the given object's docstring.
 
     See `find_autosummary_in_lines`.
@@ -258,19 +273,19 @@ def find_autosummary_in_docstring(name, module=None, filename=None):
     try:
         real_name, obj, parent, modname = import_by_name(name)
         lines = pydoc.getdoc(obj).splitlines()
-        return find_autosummary_in_lines(lines, module=name, filename=filename)  # type: ignore
+        return find_autosummary_in_lines(lines, module=name, filename=filename)
     except AttributeError:
         pass
     except ImportError as e:
         print("Failed to import '%s': %s" % (name, e))
-    except SystemExit as e:
+    except SystemExit:
         print("Failed to import '%s'; the module executes module level "
               "statement and it might call sys.exit()." % name)
     return []
 
 
 def find_autosummary_in_lines(lines, module=None, filename=None):
-    # type: (List[unicode], Any, unicode) -> List[Tuple[unicode, unicode, unicode]]
+    # type: (List[str], Any, str) -> List[Tuple[str, str, str]]
     """Find out what items appear in autosummary:: directives in the
     given lines.
 
@@ -290,9 +305,9 @@ def find_autosummary_in_lines(lines, module=None, filename=None):
     toctree_arg_re = re.compile(r'^\s+:toctree:\s*(.*?)\s*$')
     template_arg_re = re.compile(r'^\s+:template:\s*(.*?)\s*$')
 
-    documented = []  # type: List[Tuple[unicode, unicode, unicode]]
+    documented = []  # type: List[Tuple[str, str, str]]
 
-    toctree = None  # type: unicode
+    toctree = None  # type: str
     template = None
     current_module = module
     in_autosummary = False
@@ -300,7 +315,7 @@ def find_autosummary_in_lines(lines, module=None, filename=None):
 
     for line in lines:
         if in_autosummary:
-            m = toctree_arg_re.match(line)  # type: ignore
+            m = toctree_arg_re.match(line)
             if m:
                 toctree = m.group(1)
                 if filename:
@@ -308,7 +323,7 @@ def find_autosummary_in_lines(lines, module=None, filename=None):
                                            toctree)
                 continue
 
-            m = template_arg_re.match(line)  # type: ignore
+            m = template_arg_re.match(line)
             if m:
                 template = m.group(1).strip()
                 continue
@@ -316,7 +331,7 @@ def find_autosummary_in_lines(lines, module=None, filename=None):
             if line.strip().startswith(':'):
                 continue  # skip options
 
-            m = autosummary_item_re.match(line)  # type: ignore
+            m = autosummary_item_re.match(line)
             if m:
                 name = m.group(1).strip()
                 if name.startswith('~'):
@@ -332,7 +347,7 @@ def find_autosummary_in_lines(lines, module=None, filename=None):
 
             in_autosummary = False
 
-        m = autosummary_re.match(line)  # type: ignore
+        m = autosummary_re.match(line)
         if m:
             in_autosummary = True
             base_indent = m.group(1)
@@ -340,7 +355,7 @@ def find_autosummary_in_lines(lines, module=None, filename=None):
             template = None
             continue
 
-        m = automodule_re.search(line)  # type: ignore
+        m = automodule_re.search(line)
         if m:
             current_module = m.group(1).strip()
             # recurse into the automodule docstring
@@ -348,7 +363,7 @@ def find_autosummary_in_lines(lines, module=None, filename=None):
                 current_module, filename=filename))
             continue
 
-        m = module_re.match(line)  # type: ignore
+        m = module_re.match(line)
         if m:
             current_module = m.group(2)
             continue
@@ -360,8 +375,8 @@ def get_parser():
     # type: () -> argparse.ArgumentParser
     parser = argparse.ArgumentParser(
         usage='%(prog)s [OPTIONS] <SOURCE_FILE>...',
-        epilog='For more information, visit <http://sphinx-doc.org/>.',
-        description="""
+        epilog=__('For more information, visit <http://sphinx-doc.org/>.'),
+        description=__("""
 Generate ReStructuredText using autosummary directives.
 
 sphinx-autogen is a frontend to sphinx.ext.autosummary.generate. It generates
@@ -372,35 +387,38 @@ The format of the autosummary directive is documented in the
 ``sphinx.ext.autosummary`` Python module and can be read using::
 
   pydoc sphinx.ext.autosummary
-""")
+"""))
 
     parser.add_argument('--version', action='version', dest='show_version',
                         version='%%(prog)s %s' % __display_version__)
 
     parser.add_argument('source_file', nargs='+',
-                        help='source files to generate rST files for')
+                        help=__('source files to generate rST files for'))
 
     parser.add_argument('-o', '--output-dir', action='store',
                         dest='output_dir',
-                        help='directory to place all output in')
+                        help=__('directory to place all output in'))
     parser.add_argument('-s', '--suffix', action='store', dest='suffix',
                         default='rst',
-                        help='default suffix for files (default: '
-                              '%(default)s)')
+                        help=__('default suffix for files (default: '
+                                '%(default)s)'))
     parser.add_argument('-t', '--templates', action='store', dest='templates',
                         default=None,
-                        help='custom template directory (default: '
-                              '%(default)s)')
+                        help=__('custom template directory (default: '
+                                '%(default)s)'))
     parser.add_argument('-i', '--imported-members', action='store_true',
                         dest='imported_members', default=False,
-                        help='document imported members (default: '
-                              '%(default)s)')
+                        help=__('document imported members (default: '
+                                '%(default)s)'))
 
     return parser
 
 
 def main(argv=sys.argv[1:]):
     # type: (List[str]) -> None
+    sphinx.locale.setlocale(locale.LC_ALL, '')
+    sphinx.locale.init_console(os.path.join(package_dir, 'locale'), 'sphinx')
+
     app = DummyApplication()
     setup_documenters(app)
     args = get_parser().parse_args(argv)
