@@ -1,69 +1,87 @@
-# -*- coding: utf-8 -*-
 """
     sphinx.ext.viewcode
     ~~~~~~~~~~~~~~~~~~~
 
     Add links to module code in Python object descriptions.
 
-    :copyright: Copyright 2007-2015 by the Sphinx team, see AUTHORS.
+    :copyright: Copyright 2007-2019 by the Sphinx team, see AUTHORS.
     :license: BSD, see LICENSE for details.
 """
 
 import traceback
+import warnings
 
-from six import iteritems, text_type
 from docutils import nodes
 
 import sphinx
 from sphinx import addnodes
-from sphinx.locale import _
+from sphinx.deprecation import RemovedInSphinx30Warning
+from sphinx.locale import _, __
 from sphinx.pycode import ModuleAnalyzer
-from sphinx.util import get_full_modname
+from sphinx.util import get_full_modname, logging, status_iterator
 from sphinx.util.nodes import make_refnode
-from sphinx.util.console import blue
+
+if False:
+    # For type annotation
+    from typing import Any, Dict, Iterable, Iterator, Set, Tuple  # NOQA
+    from sphinx.application import Sphinx  # NOQA
+    from sphinx.config import Config  # NOQA
+    from sphinx.environment import BuildEnvironment  # NOQA
+
+logger = logging.getLogger(__name__)
 
 
 def _get_full_modname(app, modname, attribute):
+    # type: (Sphinx, str, str) -> str
     try:
         return get_full_modname(modname, attribute)
     except AttributeError:
         # sphinx.ext.viewcode can't follow class instance attribute
         # then AttributeError logging output only verbose mode.
-        app.verbose('Didn\'t find %s in %s' % (attribute, modname))
+        logger.verbose('Didn\'t find %s in %s', attribute, modname)
         return None
     except Exception as e:
         # sphinx.ext.viewcode follow python domain directives.
         # because of that, if there are no real modules exists that specified
         # by py:function or other directives, viewcode emits a lot of warnings.
         # It should be displayed only verbose mode.
-        app.verbose(traceback.format_exc().rstrip())
-        app.verbose('viewcode can\'t import %s, failed with error "%s"' %
-                    (modname, e))
+        logger.verbose(traceback.format_exc().rstrip())
+        logger.verbose('viewcode can\'t import %s, failed with error "%s"', modname, e)
         return None
 
 
 def doctree_read(app, doctree):
+    # type: (Sphinx, nodes.Node) -> None
     env = app.builder.env
     if not hasattr(env, '_viewcode_modules'):
-        env._viewcode_modules = {}
+        env._viewcode_modules = {}  # type: ignore
+    if app.builder.name == "singlehtml":
+        return
+    if app.builder.name.startswith("epub") and not env.config.viewcode_enable_epub:
+        return
 
     def has_tag(modname, fullname, docname, refname):
-        entry = env._viewcode_modules.get(modname, None)
-        try:
-            analyzer = ModuleAnalyzer.for_module(modname)
-        except Exception:
-            env._viewcode_modules[modname] = False
+        entry = env._viewcode_modules.get(modname, None)  # type: ignore
+        if entry is False:
             return
-        if not isinstance(analyzer.code, text_type):
-            code = analyzer.code.decode(analyzer.encoding)
-        else:
-            code = analyzer.code
-        if entry is None or entry[0] != code:
+
+        code_tags = app.emit_firstresult('viewcode-find-source', modname)
+        if code_tags is None:
+            try:
+                analyzer = ModuleAnalyzer.for_module(modname)
+            except Exception:
+                env._viewcode_modules[modname] = False  # type: ignore
+                return
+
             analyzer.find_tags()
-            entry = code, analyzer.tags, {}, refname
-            env._viewcode_modules[modname] = entry
-        elif entry is False:
-            return
+            code = analyzer.code
+            tags = analyzer.tags
+        else:
+            code, tags = code_tags
+
+        if entry is None or entry[0] != code:
+            entry = code, tags, {}, refname
+            env._viewcode_modules[modname] = entry  # type: ignore
         _, tags, used, _ = entry
         if fullname in tags:
             used[fullname] = docname
@@ -72,15 +90,20 @@ def doctree_read(app, doctree):
     for objnode in doctree.traverse(addnodes.desc):
         if objnode.get('domain') != 'py':
             continue
-        names = set()
+        names = set()  # type: Set[str]
         for signode in objnode:
             if not isinstance(signode, addnodes.desc_signature):
                 continue
             modname = signode.get('module')
             fullname = signode.get('fullname')
             refname = modname
-            if env.config.viewcode_import:
-                modname = _get_full_modname(app, modname, fullname)
+            if env.config.viewcode_follow_imported_members:
+                new_modname = app.emit_firstresult(
+                    'viewcode-follow-imported', modname, fullname,
+                )
+                if not new_modname:
+                    new_modname = _get_full_modname(app, modname, fullname)
+                modname = new_modname
             if not modname:
                 continue
             fullname = signode.get('fullname')
@@ -91,55 +114,61 @@ def doctree_read(app, doctree):
                 continue
             names.add(fullname)
             pagename = '_modules/' + modname.replace('.', '/')
+            inline = nodes.inline('', _('[source]'), classes=['viewcode-link'])
             onlynode = addnodes.only(expr='html')
-            onlynode += addnodes.pending_xref(
-                '', reftype='viewcode', refdomain='std', refexplicit=False,
-                reftarget=pagename, refid=fullname,
-                refdoc=env.docname)
-            onlynode[0] += nodes.inline('', _('[source]'),
-                                        classes=['viewcode-link'])
+            onlynode += addnodes.pending_xref('', inline, reftype='viewcode', refdomain='std',
+                                              refexplicit=False, reftarget=pagename,
+                                              refid=fullname, refdoc=env.docname)
             signode += onlynode
 
 
 def env_merge_info(app, env, docnames, other):
+    # type: (Sphinx, BuildEnvironment, Iterable[str], BuildEnvironment) -> None
     if not hasattr(other, '_viewcode_modules'):
         return
     # create a _viewcode_modules dict on the main environment
     if not hasattr(env, '_viewcode_modules'):
-        env._viewcode_modules = {}
+        env._viewcode_modules = {}  # type: ignore
     # now merge in the information from the subprocess
-    env._viewcode_modules.update(other._viewcode_modules)
+    env._viewcode_modules.update(other._viewcode_modules)  # type: ignore
 
 
 def missing_reference(app, env, node, contnode):
+    # type: (Sphinx, BuildEnvironment, nodes.Element, nodes.Node) -> nodes.Node
     # resolve our "viewcode" reference nodes -- they need special treatment
     if node['reftype'] == 'viewcode':
         return make_refnode(app.builder, node['refdoc'], node['reftarget'],
                             node['refid'], contnode)
 
+    return None
+
 
 def collect_pages(app):
+    # type: (Sphinx) -> Iterator[Tuple[str, Dict[str, Any], str]]
     env = app.builder.env
     if not hasattr(env, '_viewcode_modules'):
         return
-    highlighter = app.builder.highlighter
+    highlighter = app.builder.highlighter  # type: ignore
     urito = app.builder.get_relative_uri
 
-    modnames = set(env._viewcode_modules)
+    modnames = set(env._viewcode_modules)  # type: ignore
 
-#    app.builder.info(' (%d module code pages)' %
-#                     len(env._viewcode_modules), nonl=1)
-
-    for modname, entry in app.status_iterator(
-            iteritems(env._viewcode_modules), 'highlighting module code... ',
-            blue, len(env._viewcode_modules), lambda x: x[0]):
+    for modname, entry in status_iterator(
+            sorted(env._viewcode_modules.items()),  # type: ignore
+            __('highlighting module code... '), "blue",
+            len(env._viewcode_modules),  # type: ignore
+            app.verbosity, lambda x: x[0]):
         if not entry:
             continue
         code, tags, used, refname = entry
         # construct a page name for the highlighted source
         pagename = '_modules/' + modname.replace('.', '/')
         # highlight the source using the builder's highlighter
-        highlighted = highlighter.highlight_block(code, 'python', linenos=False)
+        if env.config.highlight_language in ('python3', 'default', 'none'):
+            lexer = env.config.highlight_language
+        else:
+            lexer = 'python'
+        highlighted = highlighter.highlight_block(code, lexer, linenos=False)
         # split the code into lines
         lines = highlighted.splitlines()
         # split off wrap markup from the first line of the actual code
@@ -150,14 +179,14 @@ def collect_pages(app):
         # the collected tags (HACK: this only works if the tag boundaries are
         # properly nested!)
         maxindex = len(lines) - 1
-        for name, docname in iteritems(used):
+        for name, docname in used.items():
             type, start, end = tags[name]
             backlink = urito(pagename, docname) + '#' + refname + '.' + name
             lines[start] = (
                 '<div class="viewcode-block" id="%s"><a class="viewcode-back" '
                 'href="%s">%s</a>' % (name, backlink, _('[docs]')) +
                 lines[start])
-            lines[min(end - 1, maxindex)] += '</div>'
+            lines[min(end, maxindex)] += '</div>'
         # try to find parents (for submodules)
         parents = []
         parent = modname
@@ -209,12 +238,30 @@ def collect_pages(app):
     yield ('_modules/index', context, 'page.html')
 
 
+def migrate_viewcode_import(app, config):
+    # type: (Sphinx, Config) -> None
+    if config.viewcode_import is not None:
+        warnings.warn('viewcode_import was renamed to viewcode_follow_imported_members. '
+                      'Please update your configuration.',
+                      RemovedInSphinx30Warning, stacklevel=2)
+
+
 def setup(app):
-    app.add_config_value('viewcode_import', True, False)
+    # type: (Sphinx) -> Dict[str, Any]
+    app.add_config_value('viewcode_import', None, False)
+    app.add_config_value('viewcode_enable_epub', False, False)
+    app.add_config_value('viewcode_follow_imported_members', True, False)
+    app.connect('config-inited', migrate_viewcode_import)
     app.connect('doctree-read', doctree_read)
     app.connect('env-merge-info', env_merge_info)
     app.connect('html-collect-pages', collect_pages)
     app.connect('missing-reference', missing_reference)
     # app.add_config_value('viewcode_include_modules', [], 'env')
     # app.add_config_value('viewcode_exclude_modules', [], 'env')
-    return {'version': sphinx.__version__, 'parallel_read_safe': True}
+    app.add_event('viewcode-find-source')
+    app.add_event('viewcode-follow-imported')
+    return {
+        'version': sphinx.__display_version__,
+        'env_version': 1,
+        'parallel_read_safe': True
+    }
