@@ -1,22 +1,22 @@
-# -*- coding: utf-8 -*-
 """
     test_build
     ~~~~~~~~~~
 
     Test all builders.
 
-    :copyright: Copyright 2007-2016 by the Sphinx team, see AUTHORS.
+    :copyright: Copyright 2007-2019 by the Sphinx team, see AUTHORS.
     :license: BSD, see LICENSE for details.
 """
 
-import pickle
-from docutils import nodes
-import mock
-import pytest
+import sys
 from textwrap import dedent
-from sphinx.errors import SphinxError
+from unittest import mock
 
-from util import rootdir, tempdir, path
+import pytest
+from docutils import nodes
+
+from sphinx.errors import SphinxError
+from sphinx.testing.path import path
 
 
 def request_session_head(url, **kwargs):
@@ -27,14 +27,14 @@ def request_session_head(url, **kwargs):
 
 
 @pytest.fixture
-def nonascii_srcdir(request):
+def nonascii_srcdir(request, rootdir, sphinx_test_tempdir):
     # If supported, build in a non-ASCII source dir
-    test_name = u'\u65e5\u672c\u8a9e'
-    basedir = tempdir / request.node.originalname
+    test_name = '\u65e5\u672c\u8a9e'
+    basedir = sphinx_test_tempdir / request.node.originalname
     try:
         srcdir = basedir / test_name
         if not srcdir.exists():
-            (rootdir / 'root').copytree(srcdir)
+            (rootdir / 'test-root').copytree(srcdir)
     except UnicodeEncodeError:
         srcdir = basedir / 'all'
     else:
@@ -44,39 +44,36 @@ def nonascii_srcdir(request):
             =======================
             """))
 
-        master_doc = srcdir / 'contents.txt'
-        master_doc.write_text(master_doc.text() + dedent(u"""
-                .. toctree::
+        master_doc = srcdir / 'index.txt'
+        master_doc.write_text(master_doc.text() + dedent("""
+                              .. toctree::
 
-                   %(test_name)s/%(test_name)s
-                """ % {'test_name': test_name})
-                              )
+                                 %(test_name)s/%(test_name)s
+                              """ % {'test_name': test_name}))
     return srcdir
 
 
+# note: this test skips building docs for some builders because they have independent testcase.
+#       (html, changes, epub, latex, texinfo and manpage)
 @pytest.mark.parametrize(
     "buildername",
-    [
-        # note: no 'html' - if it's ok with dirhtml it's ok with html
-        'dirhtml', 'singlehtml', 'latex', 'texinfo', 'pickle', 'json', 'text',
-        'htmlhelp', 'qthelp', 'epub2', 'epub', 'applehelp', 'changes', 'xml',
-        'pseudoxml', 'man', 'linkcheck',
-    ],
+    ['dirhtml', 'singlehtml', 'text', 'xml', 'pseudoxml', 'linkcheck'],
 )
 @mock.patch('sphinx.builders.linkcheck.requests.head',
             side_effect=request_session_head)
+@pytest.mark.xfail(sys.platform == 'win32', reason="Not working on windows")
 def test_build_all(requests_head, make_app, nonascii_srcdir, buildername):
     app = make_app(buildername, srcdir=nonascii_srcdir)
     app.build()
 
 
 def test_master_doc_not_found(tempdir, make_app):
-    (tempdir / 'conf.py').write_text('master_doc = "index"')
+    (tempdir / 'conf.py').write_text('')
     assert tempdir.listdir() == ['conf.py']
 
     app = make_app('dummy', srcdir=tempdir)
     with pytest.raises(SphinxError):
-        app.builder.build_all()
+        app.builder.build_all()  # no index.rst
 
 
 @pytest.mark.sphinx(buildername='text', testroot='circular')
@@ -85,10 +82,10 @@ def test_circular_toctree(app, status, warning):
     warnings = warning.getvalue()
     assert (
         'circular toctree references detected, ignoring: '
-        'sub <- contents <- sub') in warnings
+        'sub <- index <- sub') in warnings
     assert (
         'circular toctree references detected, ignoring: '
-        'contents <- sub <- contents') in warnings
+        'index <- sub <- index') in warnings
 
 
 @pytest.mark.sphinx(buildername='text', testroot='numbered-circular')
@@ -97,18 +94,18 @@ def test_numbered_circular_toctree(app, status, warning):
     warnings = warning.getvalue()
     assert (
         'circular toctree references detected, ignoring: '
-        'sub <- contents <- sub') in warnings
+        'sub <- index <- sub') in warnings
     assert (
         'circular toctree references detected, ignoring: '
-        'contents <- sub <- contents') in warnings
+        'index <- sub <- index') in warnings
 
 
-@pytest.mark.sphinx(buildername='dummy', testroot='image-glob')
+@pytest.mark.sphinx(buildername='dummy', testroot='images')
 def test_image_glob(app, status, warning):
     app.builder.build_all()
 
     # index.rst
-    doctree = pickle.loads((app.doctreedir / 'index.doctree').bytes())
+    doctree = app.env.get_doctree('index')
 
     assert isinstance(doctree[0][1], nodes.image)
     assert doctree[0][1]['candidates'] == {'*': 'rimg.png'}
@@ -133,7 +130,7 @@ def test_image_glob(app, status, warning):
     assert doctree[0][4][0]['uri'] == 'img.*'
 
     # subdir/index.rst
-    doctree = pickle.loads((app.doctreedir / 'subdir/index.doctree').bytes())
+    doctree = app.env.get_doctree('subdir/index')
 
     assert isinstance(doctree[0][1], nodes.image)
     sub = path('subdir')
