@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 """
     test_build_linkcheck
     ~~~~~~~~~~~~~~~~~~~~
@@ -8,7 +7,6 @@
     :copyright: Copyright 2007-2019 by the Sphinx team, see AUTHORS.
     :license: BSD, see LICENSE for details.
 """
-from __future__ import print_function
 
 import pytest
 
@@ -26,7 +24,10 @@ def test_defaults(app, status, warning):
     assert "Anchor 'does-not-exist' not found" in content
     # looking for non-existent URL should fail
     assert " Max retries exceeded with url: /doesnotexist" in content
-    assert len(content.splitlines()) == 3
+    # images should fail
+    assert "Not Found for url: http://example.com/image.png" in content
+    assert "Not Found for url: http://example.com/image2.png" in content
+    assert len(content.splitlines()) == 5
 
 
 @pytest.mark.sphinx(
@@ -34,7 +35,9 @@ def test_defaults(app, status, warning):
     confoverrides={'linkcheck_anchors_ignore': ["^!", "^top$"],
                    'linkcheck_ignore': [
                        'https://localhost:7777/doesnotexist',
-                       'http://www.sphinx-doc.org/en/1.7/intro.html#']
+                       'http://www.sphinx-doc.org/en/1.7/intro.html#',
+                       'http://example.com/image.png',
+                       'http://example.com/image2.png']
                    })
 def test_anchors_ignored(app, status, warning):
     app.builder.build_all()
