@@ -1,5 +1,5 @@
 from numbers import Integral
-from typing import Any, List, TypeVar, Union, Callable, Tuple, Optional
+from typing import Any, Dict, List, TypeVar, Union, Callable, Tuple, Optional
 
 
 def f0(x: int, y: Integral) -> None:
@@ -88,6 +88,10 @@ def f17(*, arg3, arg4):
     pass
 
 
+def f18(self, arg1: Union[int, Tuple] = 10) -> List[Dict]:
+    pass
+
+
 class Node:
     def __init__(self, parent: Optional['Node']) -> None:
         pass
